"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const luxon_1 = require("luxon");
const node_fetch_1 = __importStar(require("node-fetch"));
const url_1 = require("url");
const API_BASE_URL = "https://api.monzo.com";
const defaultOptions = {};
class Monzo {
    constructor(options) {
        const { accessToken } = { ...defaultOptions, ...options };
        this.accessToken = accessToken;
    }
    static makeOAuthURL(clientId, redirectUri, state) {
        state = state ?? Math.floor(Math.random() * 1000000).toString();
        const url = `https://auth.monzo.com/?client_id=${clientId}&redirect_uri=${encodeURIComponent(redirectUri)}&response_type=code&state=${state}`;
        return { state, url };
    }
    static async completeOAuth(clientId, clientSecret, redirectUri, code) {
        const response = await node_fetch_1.default(API_BASE_URL + "/oauth2/token", {
            method: "POST",
            body: new url_1.URLSearchParams({
                grant_type: "authorization_code",
                client_id: clientId,
                client_secret: clientSecret,
                redirect_uri: redirectUri,
                code,
            }),
        });
        if (!response.ok) {
            throw new Error(`${response.status}: ${response.statusText}`);
        }
        const { access_token, refresh_token } = (await response.json());
        return {
            accessToken: access_token,
            refreshToken: refresh_token,
        };
    }
    static async refreshOAuth(clientId, clientSecret, oldRefreshToken) {
        const response = await node_fetch_1.default(API_BASE_URL + "/oauth2/token", {
            method: "POST",
            body: new url_1.URLSearchParams({
                grant_type: "refresh_token",
                client_id: clientId,
                client_secret: clientSecret,
                refresh_token: oldRefreshToken,
            }),
        });
        if (!response.ok) {
            throw new Error(`${response.status}: ${response.statusText}`);
        }
        const { access_token, refresh_token } = (await response.json());
        return {
            accessToken: access_token,
            refreshToken: refresh_token,
        };
    }
    async listAccounts() {
        const { accounts } = await this.get("/accounts");
        return accounts.map(({ id, closed, created }) => ({
            id,
            createdOn: luxon_1.DateTime.fromISO(created)
                .toUTC(0, { keepLocalTime: true })
                .startOf("day"),
            isOpen: !closed,
        }));
    }
    async listTransactions(accountId, since) {
        const { transactions } = await this.get(`/transactions?` +
            `account_id=${accountId}&` +
            `since=${since.toISO()}&` +
            `expand[]=merchant`);
        return transactions
            .filter(({ decline_reason, amount }) => !decline_reason || amount !== 0)
            .map(({ id, created, description, amount, merchant, counterparty }) => ({
            id,
            on: luxon_1.DateTime.fromISO(created)
                .toUTC(0, { keepLocalTime: true })
                .startOf("day"),
            payee: counterparty?.preferred_name ||
                counterparty?.name ||
                merchant?.name ||
                description,
            amount,
        }));
    }
    async get(endpoint) {
        const headers = new node_fetch_1.Headers({
            Authorization: "Bearer " + this.accessToken,
        });
        const response = await node_fetch_1.default(API_BASE_URL + endpoint, { headers });
        if (!response.ok) {
            throw new Error(`${response.status}: ${response.statusText}`);
        }
        return (await response.json());
    }
}
exports.default = Monzo;
